/*
===========================================================================

Doom 3 GPL Source Code
Copyright (C) 1999-2011 id Software LLC, a ZeniMax Media company.
interaction_fs clean-room rebuilt for GPL compatibility by MH, 2012.

This file is part of the Doom 3 GPL Source Code (?Doom 3 Source Code?).

Doom 3 Source Code is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Doom 3 Source Code is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Doom 3 Source Code.  If not, see <http://www.gnu.org/licenses/>.

===========================================================================
*/

uniform sampler2D bumpImage;
uniform sampler2D lightFalloffImage;
uniform sampler2D lightProjectImage;
uniform sampler2D diffImage;
uniform sampler2D specImage;

uniform vec4 u_light_origin;
uniform vec4 u_view_origin;

uniform mat2x4 u_diffMatrix;
uniform mat2x4 u_bumpMatrix;
uniform mat2x4 u_specMatrix;

uniform mat4x4 u_projectMatrix;
uniform mat4x4 u_falloffMatrix;

uniform vec4 u_constant_diff;
uniform vec4 u_constant_spec;

varying vec4 vertexPosition;
varying vec4 vertexTexCoord;
varying vec4 vertexColor;
varying mat3x3 lightMatrix;

void main ()
{
	vec3 lightVector = normalize ((u_light_origin - vertexPosition).xyz * lightMatrix);
	vec3 viewVector =  normalize ((u_view_origin  - vertexPosition).xyz * lightMatrix);
	vec3 halfAngle =   normalize (lightVector + viewVector);

	vec4 lightProject = texture2DProj (lightProjectImage, vertexPosition * u_projectMatrix);
	vec4 lightFalloff = texture2D     (lightFalloffImage, vertexPosition * u_falloffMatrix);

	vec4 diffMap = texture2D (diffImage, vertexTexCoord * u_diffMatrix) * u_constant_diff;
	vec4 specMap = texture2D (specImage, vertexTexCoord * u_specMatrix) * u_constant_spec;
	vec3 bumpMap = texture2D (bumpImage, vertexTexCoord * u_bumpMatrix).agb * 2.0 - 1.0;

	vec4  incomingLight = lightProject * lightFalloff * dot (lightVector, bumpMap);
	float specularPower = clamp (pow (dot (halfAngle, bumpMap), 8.5), 0.0, 1.0);
	vec4  finalSpec =     (specularPower * specularPower) * specMap * 2.0;

	gl_FragColor = vec4 (((diffMap + finalSpec) * incomingLight * vertexColor).rgb, 0.0);
}

